"""Added uid column to cot table

Revision ID: 00546817d518
Revises: 807c3ca8e7d0
Create Date: 2024-10-11 20:12:20.866452

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "00546817d518"
down_revision = "807c3ca8e7d0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.add_column(sa.Column("uid", sa.String(length=255), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.drop_column("uid")

    # ### end Alembic commands ###
