"""Updated MissionLogEntry

Revision ID: 35a075531973
Revises: 2b1863a985a6
Create Date: 2025-11-12 05:01:51.094540

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "35a075531973"
down_revision = "2b1863a985a6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    with op.batch_alter_table("mission_logs", schema=None) as batch_op:
        batch_op.add_column(sa.Column("content_hashes", sa.JSON(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("mission_logs", schema=None) as batch_op:
        batch_op.drop_column("content_hashes")

    # ### end Alembic commands ###
