"""Added columns to mission_invtations table

Revision ID: f6dfc571d31c
Revises: 14e268184cc5
Create Date: 2024-10-17 18:53:11.141218

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "f6dfc571d31c"
down_revision = "14e268184cc5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    with op.batch_alter_table("mission_invitations", schema=None) as batch_op:
        batch_op.add_column(sa.Column("creator_uid", sa.String(length=255), nullable=True))
        batch_op.add_column(sa.Column("role", sa.String(length=255), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("mission_invitations", schema=None) as batch_op:
        batch_op.drop_column("role")
        batch_op.drop_column("creator_uid")

    # ### end Alembic commands ###
