"""Made cot.sender_callsign nullable

Revision ID: 85dbe0a7dcc1
Revises: 6665d030f187
Create Date: 2025-01-30 15:19:53.796637

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "85dbe0a7dcc1"
down_revision = "6665d030f187"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.alter_column(
            "sender_callsign", existing_type=sa.VARCHAR(length=255), nullable=True
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.alter_column(
            "sender_callsign", existing_type=sa.VARCHAR(length=255), nullable=False
        )

    # ### end Alembic commands ###
