"""Added plugins table

Revision ID: f310052349cb
Revises: c9fa07339363
Create Date: 2025-04-24 02:42:00.205913

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "f310052349cb"
down_revision = "c9fa07339363"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "plugins",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=255), nullable=False),
        sa.Column("enabled", sa.Boolean(), nullable=False),
        sa.Column("author", sa.String(length=255), nullable=True),
        sa.Column("version", sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("plugins")
    # ### end Alembic commands ###
